/*
* ELV-LW-SPM Payload Parser
*
* Version: V1.0.1
*
* */

function decodeUplink(input) {
  var data = input.bytes;
  var valid = true;

  if (typeof Decoder === "function") {
    data = Decoder(data, input.fPort);
  }

  if (typeof Converter === "function") {
    data = Converter(data, input.fPort);
  }

  if (typeof Validator === "function") {
    valid = Validator(data, input.fPort);
  }

  if (valid) {
    return {
      data: data
    };
  } else {
    return {
      data: {},
      errors: ["Invalid data received"]
    };
  }
}

var tx_reason = ["Undefined","Button", "Timer", "Joined"];
var activation = ["Undefined","Activated", "Deactivated"];

/*
 * @brief   Receives the bytes transmitted from a device of the ELV-LW-SPM
 * @param   bytes:  Array with the data stream
 * @param   port:   Used TTN/TTS data port
 * @return  Decoded data from a device of the ELV-LW-SPM
 * */
function Decoder(bytes, port) {
  var decoded = {};   // Container with the decoded output

  if (port === 10) {  // The default port for app data
  
    // Tx Reason
    decoded.tx_reason = tx_reason[(bytes[0])];

    // Supply Voltage
    decoded.voltage = ((bytes[1] << 8) | bytes[2])/1000;

    // Bootloader Version
    var bl_version_major = bytes[3];
    var bl_version_minor = bytes[4];
    var bl_version_patch = bytes[5];
    decoded.bootloader_version = `${bl_version_major}.${bl_version_minor}.${bl_version_patch}`; //Build version-String from 3 previous values

    // Firmware Version
    var fw_version_major = bytes[6];
    var fw_version_minor = bytes[7];
    var fw_version_patch = bytes[8];
    decoded.firmware_version = `${fw_version_major}.${fw_version_minor}.${fw_version_patch}`;//Build version-String from 3 previous values

    // Hardware Version
    decoded.hardware_version = bytes[9] << 8 | bytes[10]; 

    // Application Settings
    decoded.sensormodus = activation[(bytes[11])];
    decoded.uplink_zyklus = bytes[12];
    
    // Sensor Data
    if(decoded.sensormodus == activation[1] && decoded.tx_reason != tx_reason[3])
    {
      decoded.temperature = (((bytes[13] << 24) | (bytes[14] << 16) | (bytes[15] << 8) | bytes[16]))/ 1000;
      decoded.humidity = (((bytes[17] << 24) | (bytes[18] << 16) | (bytes[19] << 8) | bytes[20]))/ 1000;
      decoded.mc_1p0 = ((bytes[21] << 8) + bytes[22])/100;
      decoded.mc_2p5 = ((bytes[23] << 8) + bytes[24])/100;
      decoded.mc_4p0 = ((bytes[25] << 8) + bytes[26])/100;
      decoded.mc_10p0 = ((bytes[27] << 8) + bytes[28])/100;
      decoded.nc_0p5 = ((bytes[29] << 8) + bytes[30])/100;
      decoded.nc_1p0 = ((bytes[31] << 8) + bytes[32])/100;
      decoded.nc_2p5 = ((bytes[33] << 8) + bytes[34])/100;
      decoded.nc_4p0 = ((bytes[35] << 8) + bytes[36])/100;
      decoded.nc_10p0 = ((bytes[37] << 8) + bytes[38])/100;
      decoded.typical_particle_size = ((bytes[39] << 8) + bytes[40])/100;
    }
  }
  else {
    decoded.parser_error = "Wrong Port Number";
  }

  return decoded;
}